@@ -3,36 +3,42 @@ from __future__ import unicode_literals
 
 import re
 
-import django
-from django import forms
-from django.core.urlresolvers import reverse
-from django.forms.models import formset_factory
-try:
-    from django.middleware.csrf import _get_new_csrf_key
-except ImportError:
-    from django.middleware.csrf import _get_new_csrf_string as _get_new_csrf_key
-from django.template import (
-    TemplateSyntaxError, Context
-)
-
 import pytest
 
+import django
+from django import forms
+from django.forms.models import formset_factory
+from django.template import Context, Template, TemplateSyntaxError
+from django.test.html import parse_html
 from django.utils.translation import ugettext_lazy as _
 
-from .compatibility import get_template_from_string
-from .conftest import only_uni_form, only_bootstrap3, only_bootstrap4, only_bootstrap
-from .forms import TestForm, TestFormWithMedia
 from crispy_forms.bootstrap import (
-    FieldWithButtons, PrependedAppendedText, AppendedText, PrependedText,
-    StrictButton
+    AppendedText, FieldWithButtons, PrependedAppendedText, PrependedText,
+    StrictButton,
 )
 from crispy_forms.compatibility import text_type
 from crispy_forms.helper import FormHelper, FormHelpersException
 from crispy_forms.layout import (
-    Layout, Submit, Reset, Hidden, Button, MultiField, Field
+    Button, Field, Hidden, Layout, MultiField, Reset, Submit,
 )
-from crispy_forms.utils import render_crispy_form
 from crispy_forms.templatetags.crispy_forms_tags import CrispyFormNode
+from crispy_forms.utils import render_crispy_form
+
+from .conftest import (
+    only_bootstrap, only_bootstrap3, only_bootstrap4, only_uni_form,
+)
+from .forms import SampleForm, SampleFormWithMedia, SampleFormWithMultiValueField
+
+try:
+    from django.middleware.csrf import _get_new_csrf_key
+except ImportError:
+    from django.middleware.csrf import _get_new_csrf_string as _get_new_csrf_key
+
+try:
+    from django.urls import reverse
+except ImportError:
+    # Django < 1.10
+    from django.core.urlresolvers import reverse
 
 
 def test_inputs(settings):
@@ -42,11 +48,11 @@ def test_inputs(settings):
     form_helper.add_input(Hidden('my-hidden', 'Hidden'))
     form_helper.add_input(Button('my-button', 'Button'))
 
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy form form_helper %}
     """)
-    c = Context({'form': TestForm(), 'form_helper': form_helper})
+    c = Context({'form': SampleForm(), 'form_helper': form_helper})
     html = template.render(c)
 
     assert 'button white' in html
@@ -80,13 +86,13 @@ def test_form_with_helper_without_layout(settings):
     form_helper.form_action = 'simpleAction'
     form_helper.form_error_title = 'ERRORS'
 
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy testForm form_helper %}
     """)
 
     # now we render it, with errors
-    form = TestForm({'password1': 'wargame', 'password2': 'god'})
+    form = SampleForm({'password1': 'wargame', 'password2': 'god'})
     form.is_valid()
     c = Context({'testForm': form, 'form_helper': form_helper})
     html = template.render(c)
@@ -115,12 +121,12 @@ def test_form_with_helper_without_layout(settings):
 
 
 def test_form_show_errors_non_field_errors():
-    form = TestForm({'password1': 'wargame', 'password2': 'god'})
+    form = SampleForm({'password1': 'wargame', 'password2': 'god'})
     form.helper = FormHelper()
     form.helper.form_show_errors = True
     form.is_valid()
 
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy testForm %}
     """)
@@ -146,7 +152,7 @@ def test_form_show_errors_non_field_errors():
 
 
 def test_html5_required():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.html5_required = True
     html = render_crispy_form(form)
@@ -157,14 +163,14 @@ def test_html5_required():
         assert len(re.findall(r'\brequired\b', html)) == 7
 
 
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.html5_required = False
     html = render_crispy_form(form)
 
 
 def test_media_is_included_by_default_with_uniform():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'uni_form'
     html = render_crispy_form(form)
@@ -173,7 +179,7 @@ def test_media_is_included_by_default_with_uniform():
 
 
 def test_media_is_included_by_default_with_bootstrap():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'bootstrap'
     html = render_crispy_form(form)
@@ -182,7 +188,7 @@ def test_media_is_included_by_default_with_bootstrap():
 
 
 def test_media_is_included_by_default_with_bootstrap3():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'bootstrap3'
     html = render_crispy_form(form)
@@ -191,7 +197,7 @@ def test_media_is_included_by_default_with_bootstrap3():
 
 
 def test_media_is_included_by_default_with_bootstrap4():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'bootstrap4'
     html = render_crispy_form(form)
@@ -200,7 +206,7 @@ def test_media_is_included_by_default_with_bootstrap4():
 
 
 def test_media_removed_when_include_media_is_false_with_uniform():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'uni_form'
     form.helper.include_media = False
@@ -210,7 +216,7 @@ def test_media_removed_when_include_media_is_false_with_uniform():
 
 
 def test_media_removed_when_include_media_is_false_with_bootstrap():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'bootstrap'
     form.helper.include_media = False
@@ -220,7 +226,7 @@ def test_media_removed_when_include_media_is_false_with_bootstrap():
 
 
 def test_media_removed_when_include_media_is_false_with_bootstrap3():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'bootstrap3'
     form.helper.include_media = False
@@ -230,7 +236,7 @@ def test_media_removed_when_include_media_is_false_with_bootstrap3():
 
 
 def test_media_removed_when_include_media_is_false_with_bootstrap4():
-    form = TestFormWithMedia()
+    form = SampleFormWithMedia()
     form.helper = FormHelper()
     form.helper.template_pack = 'bootstrap4'
     form.helper.include_media = False
@@ -240,7 +246,7 @@ def test_media_removed_when_include_media_is_false_with_bootstrap4():
 
 
 def test_attrs():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.attrs = {'id': 'TestIdForm', 'autocomplete': "off"}
     html = render_crispy_form(form)
@@ -293,11 +299,11 @@ def test_template_helper_access():
 
 
 def test_without_helper(settings):
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy form %}
     """)
-    c = Context({'form': TestForm()})
+    c = Context({'form': SampleForm()})
     html = template.render(c)
 
     # Lets make sure everything loads right
@@ -313,11 +319,11 @@ def test_template_pack_override_compact(settings):
     override_pack = current_pack == 'uni_form' and 'bootstrap' or 'uni_form'
 
     # {% crispy form 'template_pack_name' %}
-    template = get_template_from_string("""
+    template = Template("""
         {%% load crispy_forms_tags %%}
         {%% crispy form "%s" %%}
     """ % override_pack)
-    c = Context({'form': TestForm()})
+    c = Context({'form': SampleForm()})
     html = template.render(c)
 
     if current_pack == 'uni_form':
@@ -331,11 +337,11 @@ def test_template_pack_override_verbose(settings):
     override_pack = current_pack == 'uni_form' and 'bootstrap' or 'uni_form'
 
     # {% crispy form helper 'template_pack_name' %}
-    template = get_template_from_string("""
+    template = Template("""
         {%% load crispy_forms_tags %%}
         {%% crispy form form_helper "%s" %%}
     """ % override_pack)
-    c = Context({'form': TestForm(), 'form_helper': FormHelper()})
+    c = Context({'form': SampleForm(), 'form_helper': FormHelper()})
     html = template.render(c)
 
     if current_pack == 'uni_form':
@@ -346,18 +352,18 @@ def test_template_pack_override_verbose(settings):
 
 def test_template_pack_override_wrong():
     with pytest.raises(TemplateSyntaxError):
-        get_template_from_string("""
+        Template("""
             {% load crispy_forms_tags %}
             {% crispy form 'foo' %}
         """)
 
 
 def test_invalid_helper(settings):
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy form form_helper %}
     """)
-    c = Context({'form': TestForm(), 'form_helper': "invalid"})
+    c = Context({'form': SampleForm(), 'form_helper': "invalid"})
 
     settings.CRISPY_FAIL_SILENTLY = settings.TEMPLATE_DEBUG = False
     with pytest.raises(TypeError):
@@ -365,7 +371,7 @@ def test_invalid_helper(settings):
 
 
 def test_formset_with_helper_without_layout(settings):
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy testFormSet formset_helper %}
     """)
@@ -376,8 +382,8 @@ def test_formset_with_helper_without_layout(settings):
     form_helper.form_method = 'POST'
     form_helper.form_action = 'simpleAction'
 
-    TestFormSet = formset_factory(TestForm, extra=3)
-    testFormSet = TestFormSet()
+    SampleFormSet = formset_factory(SampleForm, extra=3)
+    testFormSet = SampleFormSet()
 
     c = Context({'testFormSet': testFormSet, 'formset_helper': form_helper, 'csrf_token': _get_new_csrf_key()})
     html = template.render(c)
@@ -400,7 +406,7 @@ def test_formset_with_helper_without_layout(settings):
 
 def test_CSRF_token_POST_form():
     form_helper = FormHelper()
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy form form_helper %}
     """)
@@ -408,7 +414,7 @@ def test_CSRF_token_POST_form():
     # The middleware only initializes the CSRF token when processing a real request
     # So using RequestContext or csrf(request) here does not work.
     # Instead I set the key `csrf_token` to a CSRF token manually, which `csrf_token` tag uses
-    c = Context({'form': TestForm(), 'form_helper': form_helper, 'csrf_token': _get_new_csrf_key()})
+    c = Context({'form': SampleForm(), 'form_helper': form_helper, 'csrf_token': _get_new_csrf_key()})
     html = template.render(c)
 
     assert "<input type='hidden' name='csrfmiddlewaretoken'" in html
@@ -417,19 +423,19 @@ def test_CSRF_token_POST_form():
 def test_CSRF_token_GET_form():
     form_helper = FormHelper()
     form_helper.form_method = 'GET'
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {% crispy form form_helper %}
     """)
 
-    c = Context({'form': TestForm(), 'form_helper': form_helper, 'csrf_token': _get_new_csrf_key()})
+    c = Context({'form': SampleForm(), 'form_helper': form_helper, 'csrf_token': _get_new_csrf_key()})
     html = template.render(c)
 
     assert "<input type='hidden' name='csrfmiddlewaretoken'" not in html
 
 
 def test_disable_csrf():
-    form = TestForm()
+    form = SampleForm()
     helper = FormHelper()
     helper.disable_csrf = True
     html = render_crispy_form(form, helper, {'csrf_token': _get_new_csrf_key()})
@@ -437,7 +443,8 @@ def test_disable_csrf():
 
 
 def test_render_hidden_fields():
-    test_form = TestForm()
+    from .utils import contains_partial
+    test_form = SampleForm()
     test_form.helper = FormHelper()
     test_form.helper.layout = Layout(
         'email'
@@ -455,12 +462,12 @@ def test_render_hidden_fields():
     assert html.count('<input') == 3
     assert html.count('hidden') == 2
 
-    assert html.count('name="password1" type="hidden"') == 1
-    assert html.count('name="password2" type="hidden"') == 1
+    assert contains_partial(html, '<input name="password1" type="hidden"/>')
+    assert contains_partial(html, '<input name="password2" type="hidden"/>')
 
 
 def test_render_required_fields():
-    test_form = TestForm()
+    test_form = SampleForm()
     test_form.helper = FormHelper()
     test_form.helper.layout = Layout(
         'email'
@@ -472,7 +479,7 @@ def test_render_required_fields():
 
 
 def test_helper_custom_template():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.template = 'custom_form_template.html'
 
@@ -481,7 +488,7 @@ def test_helper_custom_template():
 
 
 def test_helper_custom_field_template():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.layout = Layout(
         'password1',
@@ -495,7 +502,7 @@ def test_helper_custom_field_template():
 
 @only_uni_form
 def test_form_show_errors():
-    form = TestForm({
+    form = SampleForm({
         'email': 'invalidemail',
         'first_name': 'first_name_too_long',
         'last_name': 'last_name_too_long',
@@ -523,7 +530,7 @@ def test_form_show_errors():
 
 @only_uni_form
 def test_multifield_errors():
-    form = TestForm({
+    form = SampleForm({
         'email': 'invalidemail',
         'password1': 'yes',
         'password2': 'yes',
@@ -549,7 +556,7 @@ def test_multifield_errors():
 
 @only_bootstrap3
 def test_bootstrap_form_show_errors():
-    form = TestForm({
+    form = SampleForm({
         'email': 'invalidemail',
         'first_name': 'first_name_too_long',
         'last_name': 'last_name_too_long',
@@ -577,7 +584,7 @@ def test_bootstrap_form_show_errors():
 
 @only_bootstrap4
 def test_bootstrap_form_show_errors():
-    form = TestForm({
+    form = SampleForm({
         'email': 'invalidemail',
         'first_name': 'first_name_too_long',
         'last_name': 'last_name_too_long',
@@ -605,7 +612,7 @@ def test_bootstrap_form_show_errors():
 
 @only_bootstrap
 def test_error_text_inline(settings):
-    form = TestForm({'email': 'invalidemail'})
+    form = SampleForm({'email': 'invalidemail'})
     form.helper = FormHelper()
     layout = Layout(
         AppendedText('first_name', 'wat'),
@@ -617,27 +624,38 @@ def test_error_text_inline(settings):
     html = render_crispy_form(form)
 
     help_class = 'help-inline'
-    if settings.CRISPY_TEMPLATE_PACK in ['bootstrap3', 'bootstrap4']:
+    if settings.CRISPY_TEMPLATE_PACK == 'bootstrap3':
         help_class = 'help-block'
+    elif settings.CRISPY_TEMPLATE_PACK == 'bootstrap4':
+        help_class = 'invalid-feedback'
 
     matches = re.findall(
         '<span id="error_\d_\w*" class="%s"' % help_class, html, re.MULTILINE
     )
     assert len(matches) == 3
 
-    form = TestForm({'email': 'invalidemail'})
+    form = SampleForm({'email': 'invalidemail'})
     form.helper = FormHelper()
     form.helper.layout = layout
     form.helper.error_text_inline = False
     html = render_crispy_form(form)
 
-    matches = re.findall('<p id="error_\d_\w*" class="help-block"', html, re.MULTILINE)
+    if settings.CRISPY_TEMPLATE_PACK in ['bootstrap', 'bootstrap3']:
+        help_class = 'help-block'
+    elif settings.CRISPY_TEMPLATE_PACK == 'bootstrap4':
+        help_class = 'invalid-feedback'
+
+    matches = re.findall(
+        '<p id="error_\d_\w*" class="%s"' % help_class,
+        html,
+        re.MULTILINE
+    )
     assert len(matches) == 3
 
 
 @only_bootstrap3
 def test_error_and_help_inline():
-    form = TestForm({'email': 'invalidemail'})
+    form = SampleForm({'email': 'invalidemail'})
     form.helper = FormHelper()
     form.helper.error_text_inline = False
     form.helper.help_text_inline = True
@@ -651,7 +669,7 @@ def test_error_and_help_inline():
     assert help_position < error_position
 
     # Viceversa
-    form = TestForm({'email': 'invalidemail'})
+    form = SampleForm({'email': 'invalidemail'})
     form.helper = FormHelper()
     form.helper.error_text_inline = True
     form.helper.help_text_inline = False
@@ -667,7 +685,7 @@ def test_error_and_help_inline():
 
 @only_bootstrap4
 def test_error_and_help_inline():
-    form = TestForm({'email': 'invalidemail'})
+    form = SampleForm({'email': 'invalidemail'})
     form.helper = FormHelper()
     form.helper.error_text_inline = False
     form.helper.help_text_inline = True
@@ -677,11 +695,11 @@ def test_error_and_help_inline():
 
     # Check that help goes before error, otherwise CSS won't work
     help_position = html.find('<span id="hint_id_email" class="help-inline">')
-    error_position = html.find('<p id="error_1_id_email" class="help-block">')
+    error_position = html.find('<p id="error_1_id_email" class="invalid-feedback">')
     assert help_position < error_position
 
     # Viceversa
-    form = TestForm({'email': 'invalidemail'})
+    form = SampleForm({'email': 'invalidemail'})
     form.helper = FormHelper()
     form.helper.error_text_inline = True
     form.helper.help_text_inline = False
@@ -697,7 +715,7 @@ def test_error_and_help_inline():
 
 @only_bootstrap
 def test_form_show_labels():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.layout = Layout(
         'password1',
@@ -727,26 +745,30 @@ def test_form_show_labels():
 
 @only_bootstrap3
 def test_label_class_and_field_class():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.label_class = 'col-lg-2'
     form.helper.field_class = 'col-lg-8'
     html = render_crispy_form(form)
+    dom = parse_html(html)
 
-    assert '<div class="form-group"> <div class="controls col-lg-offset-2 col-lg-8"> <div id="div_id_is_company" class="checkbox"> <label for="id_is_company" class=""> <input class="checkboxinput checkbox" id="id_is_company" name="is_company" type="checkbox" />'
+    snippet = parse_html('<div class="form-group"> <div class="controls col-lg-offset-2 col-lg-8"> <div id="div_id_is_company" class="checkbox"> <label for="id_is_company" class=""> <input class="checkboxinput" id="id_is_company" name="is_company" type="checkbox" />company')
+    assert dom.count(snippet)
     assert html.count('col-lg-8') == 7
 
-    form.helper.label_class = 'col-sm-3'
-    form.helper.field_class = 'col-sm-8'
+    form.helper.label_class = 'col-sm-3 col-md-4'
+    form.helper.field_class = 'col-sm-8 col-md-6'
     html = render_crispy_form(form)
+    dom = parse_html(html)
 
-    assert '<div class="form-group"> <div class="controls col-sm-offset-3 col-sm-8"> <div id="div_id_is_company" class="checkbox"> <label for="id_is_company" class=""> <input class="checkboxinput checkbox" id="id_is_company" name="is_company" type="checkbox" />'
+    snippet = parse_html('<div class="form-group"> <div class="controls col-sm-offset-3 col-md-offset-4 col-sm-8 col-md-6"> <div id="div_id_is_company" class="checkbox"> <label for="id_is_company" class=""> <input class="checkboxinput" id="id_is_company" name="is_company" type="checkbox" />company' )
+    assert dom.count(snippet)
     assert html.count('col-sm-8') == 7
 
 
 @only_bootstrap3
 def test_template_pack():
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.template_pack = 'uni_form'
     html = render_crispy_form(form)
@@ -755,8 +777,8 @@ def test_template_pack():
 
 
 @only_bootstrap4
-def test_label_class_and_field_class():
-    form = TestForm()
+def test_label_class_and_field_class_bs4():
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.label_class = 'col-lg-2'
     form.helper.field_class = 'col-lg-8'
@@ -766,18 +788,18 @@ def test_label_class_and_field_class():
     assert '<div class="col-lg-offset-2 col-lg-8">' in html
     assert html.count('col-lg-8') == 7
 
-    form.helper.label_class = 'col-sm-3'
-    form.helper.field_class = 'col-sm-8'
+    form.helper.label_class = 'col-sm-3 col-md-4'
+    form.helper.field_class = 'col-sm-8 col-md-6'
     html = render_crispy_form(form)
 
     assert '<div class="form-group">' in html
-    assert '<div class="col-sm-offset-3 col-sm-8">' in html
+    assert '<div class="col-sm-offset-3 col-md-offset-4 col-sm-8 col-md-6">' in html
     assert html.count('col-sm-8') == 7
 
 
 @only_bootstrap4
-def test_template_pack():
-    form = TestForm()
+def test_template_pack_bs4():
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.template_pack = 'uni_form'
     html = render_crispy_form(form)
@@ -790,7 +812,7 @@ def test_passthrough_context():
     Test to ensure that context is passed through implicitly from outside of
     the crispy form into the crispy form templates.
     """
-    form = TestForm()
+    form = SampleForm()
     form.helper = FormHelper()
     form.helper.template = 'custom_form_template_with_context.html'
 
@@ -802,3 +824,24 @@ def test_passthrough_context():
     html = render_crispy_form(form, helper=form.helper, context=c)
     assert "Got prefix: foo" in html
     assert "Got suffix: bar" in html
+
+
+
+
+
+@only_bootstrap3
+def test_bootstrap3_does_add_form_control_class_to_non_multivaluefield():
+    form = SampleForm()
+    form.helper = FormHelper()
+    form.helper.template_pack = 'bootstrap3'
+    html = render_crispy_form(form)
+    assert 'form-control' in html
+
+
+@only_bootstrap3
+def test_bootstrap3_does_not_add_form_control_class_to_multivaluefield():
+    form = SampleFormWithMultiValueField()
+    form.helper = FormHelper()
+    form.helper.template_pack = 'bootstrap3'
+    html = render_crispy_form(form)
+    assert 'form-control' not in html
