@@ -1,25 +1,25 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.forms.forms import BoundField
-from django.forms.models import formset_factory
-from django.template import Context
-
 import pytest
 
-from .compatibility import get_template_from_string
-from .conftest import only_bootstrap
-from .forms import TestForm
-from crispy_forms.templatetags.crispy_forms_field import crispy_addon
+from django.forms.forms import BoundField
+from django.forms.models import formset_factory
+from django.template import Context, Template
+
 from crispy_forms.exceptions import CrispyError
+from crispy_forms.templatetags.crispy_forms_field import crispy_addon
+
+from .conftest import only_bootstrap
+from .forms import SampleForm
 
 
 def test_as_crispy_errors_form_without_non_field_errors():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ form|as_crispy_errors }}
     """)
-    form = TestForm({'password1': "god", 'password2': "god"})
+    form = SampleForm({'password1': "god", 'password2': "god"})
     form.is_valid()
 
     c = Context({'form': form})
@@ -28,11 +28,11 @@ def test_as_crispy_errors_form_without_non_field_errors():
 
 
 def test_as_crispy_errors_form_with_non_field_errors():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ form|as_crispy_errors }}
     """)
-    form = TestForm({'password1': "god", 'password2': "wargame"})
+    form = SampleForm({'password1': "god", 'password2': "wargame"})
     form.is_valid()
 
     c = Context({'form': form})
@@ -43,13 +43,13 @@ def test_as_crispy_errors_form_with_non_field_errors():
 
 
 def test_as_crispy_errors_formset_without_non_form_errors():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ formset|as_crispy_errors }}
     """)
 
-    TestFormset = formset_factory(TestForm, max_num=1, validate_max=True)
-    formset = TestFormset()
+    SampleFormset = formset_factory(SampleForm, max_num=1, validate_max=True)
+    formset = SampleFormset()
     formset.is_valid()
 
     c = Context({'formset': formset})
@@ -58,13 +58,13 @@ def test_as_crispy_errors_formset_without_non_form_errors():
 
 
 def test_as_crispy_errors_formset_with_non_form_errors():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ formset|as_crispy_errors }}
     """)
 
-    TestFormset = formset_factory(TestForm, max_num=1, validate_max=True)
-    formset = TestFormset({
+    SampleFormset = formset_factory(SampleForm, max_num=1, validate_max=True)
+    formset = SampleFormset({
         'form-TOTAL_FORMS': '2',
         'form-INITIAL_FORMS': '0',
         'form-MAX_NUM_FORMS': '',
@@ -81,7 +81,7 @@ def test_as_crispy_errors_formset_with_non_form_errors():
 
 
 def test_as_crispy_field_non_field(settings):
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ field|as_crispy_field }}
     """)
@@ -97,12 +97,12 @@ def test_as_crispy_field_non_field(settings):
 
 
 def test_as_crispy_field_bound_field():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ field|as_crispy_field }}
     """)
 
-    form = TestForm({'password1': "god", 'password2': "god"})
+    form = SampleForm({'password1': "god", 'password2': "god"})
     form.is_valid()
 
     c = Context({'field': form["password1"]})
@@ -114,11 +114,11 @@ def test_as_crispy_field_bound_field():
 
 
 def test_crispy_filter_with_form():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ form|crispy }}
     """)
-    c = Context({'form': TestForm()})
+    c = Context({'form': SampleForm()})
     html = template.render(c)
 
     assert "<td>" not in html
@@ -127,13 +127,13 @@ def test_crispy_filter_with_form():
 
 
 def test_crispy_filter_with_formset():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_tags %}
         {{ testFormset|crispy }}
     """)
 
-    TestFormset = formset_factory(TestForm, extra=4)
-    testFormset = TestFormset()
+    SampleFormset = formset_factory(SampleForm, extra=4)
+    testFormset = SampleFormset()
 
     c = Context({'testFormset': testFormset})
     html = template.render(c)
@@ -146,12 +146,12 @@ def test_crispy_filter_with_formset():
 
 
 def test_classes_filter():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_field %}
         {{ testField|classes }}
     """)
 
-    test_form = TestForm()
+    test_form = SampleForm()
     test_form.fields['email'].widget.attrs.update({'class': 'email-fields'})
     c = Context({'testField': test_form.fields['email']})
     html = template.render(c)
@@ -159,11 +159,11 @@ def test_classes_filter():
 
 
 def test_crispy_field_and_class_converters():
-    template = get_template_from_string("""
+    template = Template("""
         {% load crispy_forms_field %}
         {% crispy_field testField 'class' 'error' %}
     """)
-    test_form = TestForm()
+    test_form = SampleForm()
     field_instance = test_form.fields['email']
     bound_field = BoundField(test_form, field_instance, 'email')
 
@@ -175,7 +175,7 @@ def test_crispy_field_and_class_converters():
 
 @only_bootstrap
 def test_crispy_addon(settings):
-    test_form = TestForm()
+    test_form = SampleForm()
     field_instance = test_form.fields['email']
     bound_field = BoundField(test_form, field_instance, 'email')
 
